VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CNonClientMetrics"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' *************************************************************************
'  Copyright 1999 Karl E. Peterson
'  All Rights Reserved, http://www.mvps.org/vb
' *************************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code, non-compiled, without prior written consent.
' *************************************************************************
Option Explicit

' System parameters
Private Declare Function SystemParametersInfo Lib "USER32" Alias "SystemParametersInfoA" (ByVal uAction As Long, ByVal uParam As Long, lpvParam As Any, ByVal fuWinIni As Long) As Long
Private Const SPI_GETNONCLIENTMETRICS = 41
Private Const SPI_SETNONCLIENTMETRICS = 42

' Device capabilities
Private Declare Function GetDeviceCaps Lib "gdi32" (ByVal hDC As Long, ByVal nIndex As Long) As Long
Private Const LOGPIXELSY = 90        '  Logical pixels/inch in Y

' Used to obtain screen device context
Private Declare Function GetDesktopWindow Lib "USER32" () As Long
Private Declare Function GetDC Lib "USER32" (ByVal hWnd As Long) As Long
Private Declare Function ReleaseDC Lib "USER32" (ByVal hWnd As Long, ByVal hDC As Long) As Long

' Logical Font
Private Const LF_FACESIZE = 32
Private Const LF_FULLFACESIZE = 64

' Font Weights
Private Const FW_DONTCARE = 0
Private Const FW_THIN = 100
Private Const FW_EXTRALIGHT = 200
Private Const FW_LIGHT = 300
Private Const FW_NORMAL = 400
Private Const FW_MEDIUM = 500
Private Const FW_SEMIBOLD = 600
Private Const FW_BOLD = 700
Private Const FW_EXTRABOLD = 800
Private Const FW_HEAVY = 900

Private Type LOGFONT
   lfHeight As Long
   lfWidth As Long
   lfEscapement As Long
   lfOrientation As Long
   lfWeight As Long
   lfItalic As Byte
   lfUnderline As Byte
   lfStrikeOut As Byte
   lfCharSet As Byte
   lfOutPrecision As Byte
   lfClipPrecision As Byte
   lfQuality As Byte
   lfPitchAndFamily As Byte
   lfFaceName As String * LF_FACESIZE
End Type

Private Type NONCLIENTMETRICS
   cbSize As Long
   iBorderWidth As Long
   iScrollWidth As Long
   iScrollHeight As Long
   iCaptionWidth As Long
   iCaptionHeight As Long
   lfCaptionFont As LOGFONT
   iSMCaptionWidth As Long
   iSMCaptionHeight As Long
   lfSMCaptionFont As LOGFONT
   iMenuWidth As Long
   iMenuHeight As Long
   lfMenuFont As LOGFONT
   lfStatusFont As LOGFONT
   lfMessageFont As LOGFONT
End Type

Private m_ncm As NONCLIENTMETRICS

' ***********************************************
'  Initialization
' ***********************************************
Private Sub Class_Initialize()
   ' Initialize data structure
   m_ncm.cbSize = Len(m_ncm)
   ' Cache startup values
   Call Me.Refresh
End Sub

' ***********************************************
'  Public methods
' ***********************************************
Public Sub Refresh()
   Call SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, m_ncm, 0)
End Sub

' ***********************************************
'  Public read-only properties
' ***********************************************
Public Property Get BorderWidth() As Long
   BorderWidth = m_ncm.iBorderWidth
End Property

Public Property Get ScrollWidth() As Long
   ScrollWidth = m_ncm.iScrollWidth
End Property

Public Property Get ScrollHeight() As Long
   ScrollHeight = m_ncm.iScrollHeight
End Property

Public Property Get CaptionWidth() As Long
   CaptionWidth = m_ncm.iCaptionWidth
End Property

Public Property Get CaptionHeight() As Long
   CaptionHeight = m_ncm.iCaptionHeight
End Property

Public Property Get CaptionFont() As StdFont
   Set CaptionFont = CreateFont(m_ncm.lfCaptionFont)
End Property

Public Property Get SmCaptionWidth() As Long
   SmCaptionWidth = m_ncm.iSMCaptionWidth
End Property

Public Property Get SmCaptionHeight() As Long
   SmCaptionHeight = m_ncm.iSMCaptionHeight
End Property

Public Property Get MenuWidth() As Long
   MenuWidth = m_ncm.iMenuWidth
End Property

Public Property Get MenuHeight() As Long
   MenuHeight = m_ncm.iMenuHeight
End Property

Public Property Get MenuFont() As StdFont
   Set MenuFont = CreateFont(m_ncm.lfMenuFont)
End Property

Public Property Get StatusFont() As StdFont
   Set StatusFont = CreateFont(m_ncm.lfStatusFont)
End Property

Public Property Get MessageFont() As StdFont
   Set MessageFont = CreateFont(m_ncm.lfMessageFont)
End Property

' ***********************************************
'  Private methods
' ***********************************************
Private Function CreateFont(lf As LOGFONT) As StdFont
   Dim fnt As StdFont
   ' Extracts metrics from LOGFONT structure to
   ' create and return a new StdFont object.
   Set fnt = New StdFont
   With fnt
      .Bold = (lf.lfWeight >= FW_BOLD)
      .Charset = lf.lfCharSet
      .Italic = CBool(lf.lfItalic)
      .Name = TrimNull(lf.lfFaceName)
      .Size = HeightToPoints(lf.lfHeight)
      .Strikethrough = lf.lfStrikeOut
      .Underline = lf.lfUnderline
      .Weight = lf.lfWeight
   End With
   Set CreateFont = fnt
End Function

Private Function HeightToPoints(ByVal n As Long) As Single
   ' Standard function to convert LOGFONT lfHeight
   ' elements to their equivalent point size.
   HeightToPoints = (-72 * n) / PixelsPerInchY
End Function

Private Function PointsToHeight(ByVal n As Long) As Single
   ' Standard function to convert point size to a
   ' LOGFONT lfHeight value.
   PointsToHeight = -(n * PixelsPerInchY) / 72
End Function

Private Function PixelsPerInchY() As Long
   Dim hWnd As Long
   Dim hDC As Long
   ' Retrieves the number of pixels per logical
   ' inch in the Y-direction on screen.
   hWnd = GetDesktopWindow()
   hDC = GetDC(hWnd)
   PixelsPerInchY = GetDeviceCaps(hDC, LOGPIXELSY)
   Call ReleaseDC(hWnd, hDC)
End Function

Private Function TrimNull(ByVal StrIn As String) As String
   Dim nul As Long
   ' Truncate input string at first null.
   ' If no nulls, perform ordinary Trim.
   nul = InStr(StrIn, vbNullChar)
   Select Case nul
      Case Is > 1
         TrimNull = Left(StrIn, nul - 1)
      Case 1
         TrimNull = ""
      Case 0
         TrimNull = Trim(StrIn)
   End Select
End Function

