VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CTaskList"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' *************************************************************************
'  Copyright 1999 Karl E. Peterson
'  All Rights Reserved, http://www.mvps.org/vb
' *************************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code, non-compiled, without prior written consent.
' *************************************************************************
Option Explicit

Private Declare Function GetTopWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetNextWindow Lib "user32" Alias "GetWindow" (ByVal hWnd As Long, ByVal wFlag As Long) As Long
Private Declare Function GetWindowText Lib "user32" Alias "GetWindowTextA" (ByVal hWnd As Long, ByVal lpString As String, ByVal cch As Long) As Long
Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function IsWindowVisible Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal hWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long
Private Declare Function GetClassLong Lib "user32" Alias "GetClassLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function GetParent Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function IsIconic Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Private Declare Function LoadIcon Lib "user32" Alias "LoadIconA" (ByVal hInstance As Long, lpIconName As Any) As Long
Private Declare Function GetForegroundWindow Lib "user32" () As Long

Private Const GW_HWNDFIRST = 0
Private Const GW_HWNDNEXT = 2
Private Const GWL_STYLE = (-16)

Private Const ICON_SMALL = 0
Private Const ICON_BIG = 1

Private Const GCL_HICON = (-14)
Private Const GCL_HICONSM = (-34)

Private Const GWL_HWNDPARENT = (-8)

Private Const WM_GETICON = &H7F
Private Const WM_SETICON = &H80

Private Const SW_RESTORE = 9

Private Const IDI_WINLOGO = 32517&

' Window Styles
Private Const WS_OVERLAPPED = &H0&
Private Const WS_POPUP = &H80000000
Private Const WS_CHILD = &H40000000
Private Const WS_MINIMIZE = &H20000000
Private Const WS_VISIBLE = &H10000000
Private Const WS_DISABLED = &H8000000
Private Const WS_CLIPSIBLINGS = &H4000000
Private Const WS_CLIPCHILDREN = &H2000000
Private Const WS_MAXIMIZE = &H1000000
Private Const WS_CAPTION = &HC00000                  '  WS_BORDER Or WS_DLGFRAME
Private Const WS_BORDER = &H800000
Private Const WS_DLGFRAME = &H400000
Private Const WS_VSCROLL = &H200000
Private Const WS_HSCROLL = &H100000
Private Const WS_SYSMENU = &H80000
Private Const WS_THICKFRAME = &H40000
Private Const WS_GROUP = &H20000
Private Const WS_TABSTOP = &H10000

Private Const WS_MINIMIZEBOX = &H20000
Private Const WS_MAXIMIZEBOX = &H10000

Private Const WS_TILED = WS_OVERLAPPED
Private Const WS_ICONIC = WS_MINIMIZE
Private Const WS_SIZEBOX = WS_THICKFRAME

'   Common Window Styles
Private Const WS_OVERLAPPEDWINDOW = (WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_THICKFRAME Or WS_MINIMIZEBOX Or WS_MAXIMIZEBOX)
Private Const WS_TILEDWINDOW = WS_OVERLAPPEDWINDOW
Private Const WS_POPUPWINDOW = (WS_POPUP Or WS_BORDER Or WS_SYSMENU)
Private Const WS_CHILDWINDOW = WS_CHILD

Private Type TaskListItems
   hWnd As Long
   hIcon As Long
   hIconSm As Long
   Caption As String
End Type

Private m_Count As Long
Private m_tasks() As TaskListItems
Private m_hFlagIcon As Long
   
' ************************************************
'  Initialize/Terminate
' ************************************************
Private Sub Class_Initialize()
   ' Grab handle to default flag-waving logo icon.
   m_hFlagIcon = LoadIcon(0, ByVal IDI_WINLOGO)
End Sub

Private Sub Class_Terminate()
   '
End Sub

' ************************************************
'  Public (Read-Only) Properties
' ************************************************
Public Property Get Count() As Long
   Count = m_Count
End Property

Public Property Get hWnd(ByVal Index As Long) As Long
   If Index > 0 And Index <= m_Count Then
      hWnd = m_tasks(Index - 1).hWnd
   End If
End Property

Public Property Get hIcon(ByVal Index As Long) As Long
   If Index > 0 And Index <= m_Count Then
      If m_tasks(Index - 1).hIcon <> 0 Then
         hIcon = m_tasks(Index - 1).hIcon
      Else
         hIcon = m_hFlagIcon
      End If
   End If
End Property

Public Property Get hIconSm(ByVal Index As Long) As Long
   If Index > 0 And Index <= m_Count Then
      hIconSm = m_tasks(Index - 1).hIcon
   End If
End Property

Public Property Get Caption(ByVal Index As Long) As String
   If Index > 0 And Index <= m_Count Then
      Caption = m_tasks(Index - 1).Caption
   End If
End Property

' ************************************************
'  Public Methods
' ************************************************
Public Function Activate(ByVal Index As Long) As Boolean
   If Index > 0 And Index <= m_Count Then
      With m_tasks(Index - 1)
         If Len(.Caption) Then
            If IsIconic(.hWnd) Then
               Call ShowWindow(.hWnd, SW_RESTORE)
            End If
            AppActivate .Caption
            Activate = (GetForegroundWindow = .hWnd)
         End If
      End With
   End If
End Function

' Special thanks to Jonathan Wood <http://www.softcircuits.com>
' for pointing out that the Alt+Tab order is the same as returned
' by a GetWindow loop.
Public Sub Refresh()
   Dim hWnd As Long
   Dim hIcon As Long
   Dim hIconSm As Long
   Dim Caption As String
   Dim dwStyle As Long
   Dim nLen As Long
   Dim ValidTask As Boolean
   Const MAX_LEN = 1024
   
   ' Reset variables
   m_Count = 0
   Caption = Space$(MAX_LEN)
   ' Grab first window
   hWnd = GetTopWindow(0)
   Do While hWnd
      ' Assume it's not a valid task window.
      ValidTask = False
      ' Get this window's style bits
      dwStyle = GetWindowLong(hWnd, GWL_STYLE)
      
      ' Make sure window is visible and ownerless
      If IsWindowVisible(hWnd) Then
         If GetParent(hWnd) = 0 Then
            If GetWindowLong(hWnd, GWL_HWNDPARENT) = 0 Then
               ' This is probably a good one
               ValidTask = True

               ' Weed out tooltips -- special case
               Call GetClassName(hWnd, Caption, Len(Caption))
               If InStr(Caption, "VBBubble") = 1 Then
                  ValidTask = False
               End If

               ' Desktop window given Progman classname
               ' to keep older apps from failing
               If InStr(Caption, "Progman") = 1 Then
                  If (dwStyle And WS_OVERLAPPEDWINDOW) = 0 Then
                     ValidTask = False
                  End If
               End If
            End If
         End If
      End If
      
      
      If ValidTask Then
         ' Get the hIcon(s) and Caption
         hIcon = SendMessage(hWnd, WM_GETICON, ICON_BIG, ByVal 0&)
         If hIcon = 0 Then hIcon = GetClassLong(hWnd, GCL_HICON)
         hIconSm = SendMessage(hWnd, WM_GETICON, ICON_SMALL, ByVal 0&)
         If hIconSm = 0 Then hIconSm = GetClassLong(hWnd, GCL_HICONSM)
         nLen = GetWindowText(hWnd, Caption, MAX_LEN)
         ' Add new item to array *if* we have a caption.
         If nLen Then
            ReDim Preserve m_tasks(0 To m_Count) As TaskListItems
            m_tasks(m_Count).hWnd = hWnd
            m_tasks(m_Count).hIcon = hIcon
            m_tasks(m_Count).hIconSm = hIconSm
            m_tasks(m_Count).Caption = Left$(Caption, nLen)
            ' Bump counter
            m_Count = m_Count + 1
         End If
      End If
      ' Get next window and move on
      hWnd = GetNextWindow(hWnd, GW_HWNDNEXT)
   Loop
End Sub

